Methods and tools for performing multistep-ahead time series forecasting using conformal prediction methods including classical conformal prediction, adaptive conformal prediction, conformal PID (Proportional-Integral-Derivative) control, and autocorrelated multistep-ahead conformal prediction. The methods were described by Wang and Hyndman (2024) tools:::Rd_expr_doi("10.48550/arXiv.2410.13115").
Maintainer: Xiaoqian Wang Xiaoqian.Wang@amss.ac.cn (ORCID) [copyright holder]
Authors:
Rob Hyndman Rob.Hyndman@monash.edu (ORCID)